﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Media;

namespace SLXnaGame {

    public partial class GamePage : PhoneApplicationPage {

        UIElementRenderer elementRenderer;
        TextBlock textBlock;

        ContentManager content;
        GameTimer timer;
        SpriteBatch spriteBatch;
        GraphicsDevice graphicsDevice;
        SpriteFont font;

        public GamePage() {
            InitializeComponent();
            this.SupportedOrientations = SupportedPageOrientation.Landscape;
            this.LayoutUpdated += GamePage_LayoutUpdated;
            MediaPlayer.Stop();
            graphicsDevice = SharedGraphicsDeviceManager.Current.GraphicsDevice;
            // 从应用程序中获取内容管理器
            content = (Application.Current as App).Content;
            // 为此页面创建计时器
            timer = new GameTimer();
            timer.UpdateInterval = TimeSpan.FromTicks(333333);
            timer.Update += Update;
            timer.Draw += Draw;
        }

        protected override void OnNavigatedTo(NavigationEventArgs e) {
            // 设置图形设备的共享模式以启用 XNA 呈现
            graphicsDevice.SetSharingMode(true);
            // 创建可以用来绘制纹理的新 SpriteBatch。
            spriteBatch = new SpriteBatch(graphicsDevice);
            // TODO: 使用 this.content 在此处加载游戏内容
            textBlock = new TextBlock() { Text = "Hello Game, I,m Silverlight TextBlock" };
            LayoutRoot.Children.Add(textBlock); //将textBlock添加进Canvas画布中
            Canvas.SetLeft(textBlock, 10); Canvas.SetTop(textBlock, 20); //设置textBlock在画布中的绝对位置
            font = content.Load<SpriteFont>("Font/MyFont");

            timer.Start();
            base.OnNavigatedTo(e);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e) {
            timer.Stop();
            // 设置图形设备的共享模式以禁用 XNA 呈现
            graphicsDevice.SetSharingMode(false);
            base.OnNavigatedFrom(e);
        }

        /// <summary>
        /// 允许页面运行逻辑，如更新环境、
        /// 检查冲突、收集输入和播放音频。
        /// </summary>
        private void Update(object sender, GameTimerEventArgs e) {

        }

        /// <summary>
        /// 允许页面绘制自身。
        /// </summary>
        private void Draw(object sender, GameTimerEventArgs e) {
            graphicsDevice.Clear(Color.CornflowerBlue);
            elementRenderer.Render(); //通过elementRenderer呈现Silverlight中的UElement
            spriteBatch.Begin();
            spriteBatch.Draw(elementRenderer.Texture, Vector2.Zero, Color.White); //通过XNA的形式将elementRenderer整体绘制出来
            spriteBatch.DrawString(font, "Hello Game, I,m XNA Font", new Vector2(20, 55), Color.Yellow); //第三个参数代表绘制的绝对位置
            spriteBatch.End();
        }

        /// <summary>
        /// 创建一个可以被XNA绘制的Silverlight-UI展示器UIElementRenderer
        /// </summary>
        void GamePage_LayoutUpdated(object sender, EventArgs e) {
            if (ActualWidth > 0 && ActualHeight > 0 && elementRenderer == null) {
                elementRenderer = new UIElementRenderer(this, (int)ActualWidth, (int)ActualHeight);
            }
        }
    }
}